#!/usr/bin/env python
#
# AmarokFS startup script for Amarok
# Copyright (c) 2007 Francois Aucamp <faucamp@fedoraproject.org>
#
# This script simply provides a shortcut for starting AmarokFS ("Amarok Full
# Screen") from within Amarok itself.
# It was written for the Fedora "amarokFS" package.
#
#############################################################################
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#############################################################################

from commands import getoutput
from os import system, getenv
from signal import signal
from sys import exit, stdin


def chooseAmarokFSCommand():
	""" Presents the user with a file input dialog in order to browse
	    to the amarokFS executable's location
	
	Returns:
		The chosen application (full path as a string), or None if
		the user cancelled the operation
	"""
	cmd = getoutput('kdialog --title "Please select the \'amarokFS\' executable to run" --getopenfilename "%s" "amarokFS"' % getenv('HOME'))
	if cmd == '':
		cmd = None
	return cmd
	

def verifyAmarokFSCommand(cmd):
	""" Verify amarokFS installation
	
	Checks if AmarokFS is available, and prompts the user for the appropriate command if not found.
	
	Parameters:
		cmd : string - the amarokFS command to run/verify
	
	Returns:
		The verified command to run to start amarokFS (as a string)
	"""
	if system('which %s' % cmd):
		if not system('kdialog --title "AmarokFS Warning" --warningyesno "Could not find \'amarokFS\' executable.\nWould you like to locate it yourself?"'):
			cmd = chooseAmarokFSCommand()
		else:
			cmd = None
	return cmd


def loadAmarokFSCommand():
	""" Load previously-saved configuration data
	
	Returns:
		The command to run to start amarokFS (as a string)
	"""
	try:
		f = open("amarokFS.command", "r")
		cmd = f.readline()[:-1]
		f.close()
	except IOError:
		cmd = 'amarokFS'
	
	return cmd


def storeAmarokFSCommand(cmd):
	""" Stores the configuration data """
	try:
		f = open("amarokFS.command", "w")
		f.write('%s\n' % cmd)
		f.close()
	except IOError:
		system('dcop amarok playlist popupMessage "Could not store AmarokFS Startup Script configuration."')


def main():
	""" Main application loop """
	# Test Amarok DCOP connection
	if system('dcop amarok'):
		print 'This script is meant to be called from within Amarok.\n'
		exit(1)
	
	cmd = loadAmarokFSCommand()
	cmd = verifyAmarokFSCommand(cmd)
	
	if not cmd:
		system('kdialog --title "AmarokFS Error" --error "AmarokFS must be installed for this script to function."')
		exit(0)
	
	# Add "amarokFS" custom menu entry
	if system('dcop amarok script addCustomMenuItem "AmarokFS" "Run Amarok Full Screen"'):
		system('kdialog --error "Could not create \'amarokFS\' menu entry."')
		exit(1)
	
	# Store the amarokFS command that was selected
	storeAmarokFSCommand(cmd)
	
	while 1:
		# Read notifications
		notification = stdin.readline()
		
		if notification != '':
			if 'customMenuClicked' in notification:
				if 'AmarokFS' in notification:
					if (system(cmd)) != 0:
						system('dcop amarok playlist popupMessage "There was an error launching AmarokFS.\nPlease check your installation and configuration."')
						stopScript(None, None)
						exit(0)
			elif 'configure' in notification:
				newCmd = chooseAmarokFSCommand()
				if newCmd is not None:
					cmd = newCmd
				del newCmd
		else:
			exit(0)


def stopScript(signum, stackframe):
	""" Executed when script is stopped by the user """
	system('dcop amarok script removeCustomMenuItem "AmarokFS" "Run Amarok Full Screen"')


def forceStop(signum, stackframe):
	""" Handler to stop the script when Amarok crashes """
	exit(1)

if __name__ == "__main__":
	signal(6, stopScript)
	signal(15, stopScript)
	main()
